import java.util.*;

/**
 * Light-cycle contest
 *
 * This program inputs the movements of two light cycles, and determines
 * which wins.
 *
 * The input data is read and stored in the following global variables:
 *
 * int width, height	driving region width and height
 * int xStart[p]		initial x-coordinate of player p
 * int yStart[p]		initial y-coordinate of player p
 * int nSeg[p]			number of motion segments for player p
 * char turn[p][i]		i-th turn ('L', 'R', or 'S') for player p
 * int dist[p][i]		i-th distance for player p
 *
 * The output data is read and stored in the following global variables:
 *
 * int endTime			time at which game ends
 * int xFinal[p]		x-coordinate of player p at end
 * int yFinal[p]		y-coordinate of player p at end
 * boolean isAlive[p]	true if player[p] is alive at end
 *
 * The function playGame() simulates the game and stores the results in
 * the output variables.
 */
public class LightCycle {

	// global inputs (arrays are allocated in main program)

	static private int width;				// playing region width and height
	static private int height;

	static private int xStart[];			// player[p] start coordinates
	static private int yStart[];

	static private int nSeg[];				// number of segments for player p
	static private char turn[][];			// turns ('L', 'R', or 'S')
	static private int dist[][];			// distance after turning

	// global outputs (arrays are allocated in main program)
	
	static private int endTime;				// time at which game ends
	static private int xFinal[];			// x-coordinate of player p at end
	static private int yFinal[];			// y-coordinate of player p at end
	static private boolean isAlive[];		// true if player[p] is alive
	
	/** 
	 *  main program
	 */
	public static void main(String[] args) throws Exception {
		Scanner scanner = new Scanner(System.in);

        int numCases = scanner.nextInt();

        for(int i = 0; i < numCases; i++) {
            System.out.println("----------------------------------------");
            getInputs(scanner);					// get inputs
            summarizeInput();				// summarize inputs
            playGame();						// play game and get results
            summarizeOutput();				// summarize output
        }
	}

	private static void getInputs(Scanner scanner) {
		xStart = new int[2];			// allocate arrays
		yStart = new int[2];
		nSeg = new int[2];
		turn = new char[2][];
		dist = new int[2][];
		xFinal = new int[2];
		yFinal = new int[2];
		isAlive = new boolean[2];
										// read input
		width = scanner.nextInt();		// input width and height
		height = scanner.nextInt();

		for (int p = 0; p < 2; p++) {	// input player[p] information
			xStart[p] = scanner.nextInt();
			yStart[p] = scanner.nextInt();
			if (xStart[p] < 0 || xStart[p] > width
			 || yStart[p] < 0 || yStart[p] > height) {
				System.out.println("Illegal start position");
			}
			nSeg[p] = scanner.nextInt();
			if (nSeg[p] < 0) {
				System.out.println("Illegal number of segments");
			}
			turn[p] = new char[nSeg[p]];
			dist[p] = new int[nSeg[p]];
			for (int i = 0; i < nSeg[p]; i++) {
				char t = scanner.next().toCharArray()[0];
				if (t != 'S' && t != 'L' && t != 'R') {
					System.out.println("Invalid turn direction");
				}
				turn[p][i] = t;
				dist[p][i] = scanner.nextInt();
				if (dist[p][i] <= 0) {
					System.out.println("Distance must be positive");
				}
			}
		}
	}

	private static void summarizeInput()		// summarize input
	{
		System.out.println("Width = " + width + " Height = " + height);
		for (int p = 0; p < 2; p++) {
			System.out.println("Player " + p + ":");
			System.out.println("  Start: (" + xStart[p] + ", " + yStart[p] + ")");
			System.out.print("  Segs: ");
			for (int i = 0; i < nSeg[p]; i++) {
				System.out.print(" " + turn[p][i] + " " + dist[p][i]);
			}
			System.out.println();
		}
	}

	private static void summarizeOutput()		// summarize output
	{
		System.out.println("Game ends at time " + endTime);

		for (int p = 0; p < 2; p++) {			// output player results
			System.out.print("Player " + p + " final position: " +
				"(" + xFinal[p] + ", " + yFinal[p] + ")");
			if (isAlive[p]) {
				System.out.println(" --> alive");
			}
			else {
				System.out.println(" --> dead");
			}
		}
	}

	/** playGame
	 *  
	 * Simulates the light-cycle game, based on the global input
	 * variables (width, height, xStart, yStart, nSeg, turn, dist) and
	 * stores the final results in the global output data (endTime,
	 * xFinal, yFinal, isAlive).
	 */
	private static void playGame()
	{
		/*------------------- INSERT YOUR CODE HERE -----------------------*/

/*vvvvvvvvvvvvvvvvvvvv DELETE FROM STUDENT CODE vvvvvvvvvvvvvvvvvvvvvv*/
		/*--------------------------------------------------------------
		 * This method works by constructing a two dimensional boolean
		 * array representing the playing region. For each of the two
		 * players, we simulate time and fill in each player's motion.
		 * When we come to the end of a segment, we update the direction
		 * according to the turn information. A player dies when it
		 * visits a cell that lies outside the playing region or visits
		 * a previously visited cell.
		 *
		 * To handle simultaneous death due to a head-to-head collision
		 * we add a separate check that the two players land on the same
		 * cell.
		 *
		 * Warning: This procedure modifies dist[][].
		 *--------------------------------------------------------------
		 */
		boolean visited[][] = new boolean[width+1][height+1];

		for (int i = 0; i < width; i++) {		// initialize playing region
			for (int j = 0; j < height; j++) {
				visited[i][j] = false;
			}
		}

		int x[] = new int[2];					// player's current position
		int y[] = new int[2];
		char dir[] = new char[2];				// current direction ('NESW')
		int seg[] = new int[2];					// current segment index
		int time = 0;							// current time
		boolean isDone = false;					// is the game done?

		for (int p = 0; p < 2; p++) {			// initialize
			x[p] = xStart[p];
			y[p] = yStart[p];
			visited[x[p]][y[p]] = true;
			dir[p] = 'E';
			seg[p] = -1;
			isAlive[p] = true;
		}

		for (int p = 0; p < 2; p++) {			// check initial config
			if (x[p] < 0 || x[p] > width		// dead even at start?
			 || y[p] < 0 || y[p] > height
			 || (x[p] == x[1-p] && y[p] == y[1-p])) {
				isAlive[p] = false;
				endTime = time;
				isDone = true;
			}
		}

		while (!isDone) {						// main simulation loop
		    time++;								// increment time
			for (int p = 0; p < 2; p++) {		// move player p
												// start a new segment?
				if (seg[p] == -1 || dist[p][seg[p]] == 0) {
					seg[p]++;					// advance to next segment
					if (seg[p] >= nSeg[p]) {	// out of segments?
						isDone = true;
						break;
					}
					switch (turn[p][seg[p]]) {	// update direction
						case 'S':				// straight
							break;
						case 'L':				// left turn
							switch (dir[p]) {
								case 'N': dir[p] = 'W'; break;
								case 'E': dir[p] = 'N'; break;
								case 'S': dir[p] = 'E'; break;
								case 'W': dir[p] = 'S'; break;
							}
							break;
						case 'R':				// right turn
							switch (dir[p]) {
								case 'N': dir[p] = 'E'; break;
								case 'E': dir[p] = 'S'; break;
								case 'S': dir[p] = 'W'; break;
								case 'W': dir[p] = 'N'; break;
							}
							break;
					}
				}

				switch (dir[p]) {				// update position
					case 'N': y[p]++; break;
					case 'E': x[p]++; break;
					case 'S': y[p]--; break;
					case 'W': x[p]--; break;
				}
				dist[p][seg[p]]--;
												// check if player dies
				if (x[p] < 0 || x[p] > width	// out of bounds?
				 || y[p] < 0 || y[p] > height
				 || visited[x[p]][y[p]]) {		// already visited?
					isAlive[p] = false;			// player p dies
					endTime = time;
					isDone = true;
					// The following test is needed in case of a cycle-cycle
					// collision. Player 0 moves first, and so he is not
					// aware of the collision until player 1 moves.
					if (p == 1 && x[1] == x[0] && y[1] == y[0]) {
						isAlive[0] = false;		// player 0 also dead
					}
				}
				else {
					visited[x[p]][y[p]] = true;	// mark as visited
				}
			}
		}
		for (int p = 0; p < 2; p++) {			// save final positions
			xFinal[p] = x[p];
			yFinal[p] = y[p];
		}
		endTime = time;
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^ END OF DELETION ^^^^^^^^^^^^^^^^^^^^^^^^*/
	}
}
