/*
Tron RotationMaze
=================

Each input case looks like this:
nx ny
XXXXXXXXXXX
XsX...X...X
X.X.X.X.X.X
X.X.X.X.X.X
X...X...XeX
XXXXXXXXXXX

where nx and ny are integers giving the number of columns and rows in the maze,
respectively. Following these numbers are ny lines, each of length nx. Each
character in these lines represents a cell of the maze. A "." character
represents an empty part of the maze (over which the ball can roll). The "s"
and "e" characters represent the starting and goal (end) positions of the ball,
respectively. Leading and trailing whitespace is removed. Any other character
represents an obstacle (a wall) that the ball cannot pass through. 

Notes:
  (1) It is possible for the ball to fall off the board (i.e. the maze does not
  have to be enclosed).

  (2) The start and end points do not need to be at the edge of the board.

  (3) If there is more than one sequence of rotations, return the shortest one.

  (4) If there are several shortest sequences, return the one that would come
  alphabetically first if viewed as a word.

  (5) If no solution exists, return NONE.
  
  (6) The board starts in the orientation as it is shown in the file (with
  gravity acting down the screen). The ball should be allowed to fall from its
  starting place (if it can) before any rotations occur.
*/

import java.util.Scanner;
import java.util.LinkedList;
import java.util.Set;
import java.util.HashSet;
import java.io.File;
import java.io.FileNotFoundException;

public class RotationMaze {
	
	/** The type of cell at a given location in the maze **/
	public enum BlockType {EMPTY, START, END, WALL};
	
    /* ------------------- ANSWER CODE START ---------------------*/
	
	/** Current direction of the board **/
	public enum Orientation {NORTH, SOUTH, EAST, WEST};
	
	public enum Direction {LEFT, RIGHT};
	
	/** A location on the maze board **/
	public static class MazeState {
		public int x, y; /** the position **/
		public Orientation or; /** orientation of the board **/
		public MazeState parent; /** the state that lead to this state **/
		public Direction rotation; /** the rotation that lead to this state **/
		
		public MazeState(int newx, int newy, Orientation newor) { 
			x = newx; 
			y = newy;
			or = newor; 
			parent = null;  
			rotation = null; 
		}
		
		/** True unless this is marked invalid**/
		public boolean isValid() { return x >= 0 && y >= 0; }
		
		/** Autogenerated by eclipse **/
		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((or == null) ? 0 : or.hashCode());
			result = prime * result + x;
			result = prime * result + y;
			return result;
		}
		
		/** Autogenerated by eclipse **/
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || getClass() != obj.getClass())
				return false;
			MazeState other = (MazeState) obj;
			if (or != other.or || x != other.x || y != other.y)
				return false;
			return true;
		}
	}
	
    /** Return the farthest non-wall location you can reach by falling in the
     * given orientation **/
	public static MazeState fall(
        BlockType [][] Maze, 
        int nx, int ny, 
        int x, int y, 
        Orientation or) 
    {
		assert Maze[x][y] != BlockType.WALL;
		
		int dx=0, dy=0;
		switch (or)
		{
			case NORTH: dx = 0;  dy = +1; break;
			case SOUTH: dx = 0;  dy = -1; break;
			case EAST:  dx = -1; dy = 0; break;
			case WEST:  dx = +1; dy = 0; break;
		}
		
		int lastx = x, lasty = y;
		while (x >= 0 && x < nx && y >= 0 && y < ny) 
		{
			if (Maze[x][y] == BlockType.WALL) return new MazeState(lastx, lasty, or);
			
			lastx = x; 
			lasty = y;
			x += dx; 
			y += dy;
		}
		
		// return an out of bounds location if you fall off the board
		return new MazeState(-1,-1, or);
	}
	
	/** Rotate the orientation **/
	public static Orientation rotate(Orientation or, Direction d) {
		if (d == Direction.RIGHT)
		{
			switch (or)
			{
			case NORTH: return Orientation.WEST;
			case WEST: return Orientation.SOUTH;
			case SOUTH: return Orientation.EAST;
			case EAST: return Orientation.NORTH;
			}
		}
		else 
		{
			switch (or)
			{
			case NORTH: return Orientation.EAST;
			case SOUTH: return Orientation.WEST;
			case EAST: return Orientation.SOUTH;
			case WEST: return Orientation.NORTH;
			}
		}
		assert false;
		return null; // never reached
	}
	
	/* ------------------- ANSWER CODE END ---------------------*/
	
	/** Find the shortest path through the maze **/
	public static String solveRotationMaze(
        BlockType [][] Maze, 
        int nx, int ny, 
        int startx, int starty) 
    {
        /* ------------------- INSERT CODE HERE ---------------------*/
		Set<MazeState> VisitedStates = new HashSet<MazeState>();
		LinkedList<MazeState> Frontier = new LinkedList<MazeState>();
		MazeState Start = fall(Maze, nx, ny, startx, starty, Orientation.NORTH);
		VisitedStates.add(Start);
		Frontier.addFirst(Start);
		
		while (!Frontier.isEmpty()) 
		{
			MazeState s = Frontier.removeLast();
			
			if (Maze[s.x][s.y] == BlockType.END)
			{
				LinkedList<String> I = new LinkedList<String>();
				while(s.rotation != null)
				{
					I.addFirst((s.rotation == Direction.LEFT)?"L":"R");
					s = s.parent;
				}
				
				StringBuilder sb = new StringBuilder();
				for (String i : I) sb.append(i);
				return sb.toString();
			}
			else
			{
				for (Direction d : Direction.values()) 
				{
					MazeState newlocR = fall(Maze, nx, ny, s.x, s.y, rotate(s.or, d));
					newlocR.parent = s;
					newlocR.rotation = d;
					if (newlocR.isValid() && ! VisitedStates.contains(newlocR)) 
					{
						Frontier.addFirst(newlocR);
						VisitedStates.add(newlocR);
					}
				}
			}
		}
		return "NONE";
        /* -------------------- END OF INSERTION --------------------*/
	}


	public static void main(String[] args) throws FileNotFoundException {
        //Scanner sc = new Scanner(new File("02.in"));
		Scanner sc = new Scanner(System.in);
        int numCases = sc.nextInt();

        for(int i = 0; i < numCases; i++) 
        {
        	// read the dimensions of the maze
            int nx = sc.nextInt();
            int ny = sc.nextInt();
            sc.nextLine();

            // create a 2-d array to hold the maze
            BlockType [][] Maze = new BlockType [nx][ny];
            int startx=-1, starty=-1;
            for(int j = 0; j < ny; j++)
            {
            	String line = sc.nextLine();
            	line = line.trim();
            	assert line.length() == nx;
            	
            	// scan the current line (j) of the maze
            	for (int c = 0; c < nx; c++)
            	{
            		switch(line.charAt(c))
            		{
            			case '.': Maze[c][j] = BlockType.EMPTY; break;
            			case 's': Maze[c][j] = BlockType.START; startx=c; starty=j; break;
            			case 'e': Maze[c][j] = BlockType.END; break;
            			default: Maze[c][j] = BlockType.WALL; break;
            		}
            	}
            }

            assert startx >= 0 && starty >= 0;
            System.out.println("Case " + (i+1) + ": " + solveRotationMaze(Maze, nx, ny, startx, starty));
        }
	}
}

