import java.util.*;

/**
 * Making change with infinitely many types of coins
 *
 * This program determines whether it is possible to generate a sum of
 * coins of the form 1/m (excluding some forbidden value) that sum to 1,
 * where each m is a positive integer and each coin may be used at most
 * a given number of times.
 *
 * The following function computes the ways of making coins, sorts them
 * lexicographically, and returns an array containing all the solutions.
 *
 *	private static int[][] makeChange(
 *		int n,			// number of coins to use
 *		int r, 			// maximum number of repetitions per coin
 *		int[] F) 		// forbidden coins
 *
 */
public class TronChangeOriginal {
	
	static double TINY = 1E-13;			// a very small number
	
	/** 
	 *  main program
	 */
	public static void main(String[] args) throws Exception {
										// read input
		Scanner scanner = new Scanner(System.in);
        int numCases = scanner.nextInt();

        for(int t = 0; t < numCases; t++) {
            int n = scanner.nextInt();		// input number of reciprocals
            if (n <= 0) {
                System.out.println("Error: Need at least 1 coin");
                System.exit(0);
            }
            int r = scanner.nextInt();		// input number of repetitions
            if (r < 0) {
                System.out.println("Error: Repetition count must be nonnegative");
                System.exit(0);
            }

            int f = scanner.nextInt(); 		// number of forbidden values
            int[] F = new int[f];			// list of forbidden values
            for (int i = 0; i < f; i++) {	// input forbidden
                F[i] = scanner.nextInt();
                if (F[i] <= 0) {
                    System.out.println("Error: Forbidden values must be positive");
                    System.exit(0);
                }
            }

            summarizeInput(n, r, F);		// print input summary
            int[][] R = makeChange(n, r, F); // do it
            summarizeOutput(n, r, F, R);	// print output summary
        }
	}
	
	/** 
	 *  summarize input
	 */
	private static void summarizeInput(	// print input summary
		int n,							// number of coins
		int r, 							// max repetitions per coin
		int[] F) 						// forbidden coins
	{
		System.out.println("Number of coins: " + n);
		if (r == 0) {
			System.out.println("Coins may be repeated any number of times");
		}
		else {
			System.out.println("Each coin may be repeated at most " + r + " times");
		}
		if (F.length == 0) {
			System.out.println("No forbidden coins");
		}
		else {
			System.out.print("Forbidden coins:");
			for (int i = 0; i < F.length; i++) {
				System.out.print(" " + F[i]);
			}
			System.out.println();
		}
	}
	
	/** 
	 *  summarize output and check for correctness
	 */
	private static void summarizeOutput(	// print output summary
		int n,								// number of coins
		int r, 								// max repetitions per coin
		int[] F, 							// forbidden coins
		int[][] R)							// final result
	{
		if (R.length == 0) {			// no solution
			System.out.println("No solutions found");
		}
    	else {							// print & check solution
			for (int i = 0; i < R.length; i++) {
				int[] S = R[i];		// one solution
				double sum = 0;			// sum of values
				int prev = 0;			// previous value
				int min = 0;			// minimum value
				boolean isIncr = true;	// sequence increases
				int reps = 0;			// repetitions
				int mreps = 0;			// max number of repetitions
										// print solution
				for (int j = 0; j < S.length; j++) {
					if (j != 0) System.out.print(" ");
					System.out.print(S[j]);
					sum += 1/((double) S[j]); // add to sum
										// update statistics
					min = Math.min(S[j], min);
					if (S[j] < prev) isIncr = false;
					if (S[j] == prev) {	// repeated value?
						reps++;			// update number of reps
						mreps = Math.max(reps, mreps); // and max reps
					}
					else { 				// starting a new sequence
						prev = S[j];
						reps = 0; 
					}
				}
				double error = sum - 1.0;
				if (Math.abs(error) > TINY) {
					System.out.print(": error = " + (sum - 1.0));
				}
				System.out.println();
										// check for errors
				if (S.length != n) {
					System.out.println("Wrong number of coins: " + S.length);
					System.exit(0);
				}
				if (Math.abs(sum - 1) > TINY) {
					System.out.println("Incorrect sum: " + sum);
					System.exit(0);
				}
				if (min < 0) {
					System.out.println("Negative value coin: " + min);
					System.exit(0);
				}
				if (r > 0 && mreps > r) {
					System.out.println("Too many repetitions: " + mreps);
					System.exit(0);
				}
				if (!isIncr) {
					System.out.println("Values not given in increasing order");
					System.exit(0);
				}
			}
			System.out.println(R.length + " solutions found");
    	}
	}

	/** makeChange
	 *
	 * Given the parameters n, r, and F, generates a two dimensional
	 * sorted array of solutions.
	 *
	 * Each row of this array is an array of n integers, such that the
	 * sum of reciprocals of this array sum to 1. If r != 0, then no
	 * coin may be used more than r times. Each element of F is an
	 * integer that is forbidden, meaning that 1/F[i] cannot be used in
	 * the sum. The number of forbidden values is F.length.
	 *
	 */
	private static int[][] makeChange(
		int n,			// number of coins to use
		int r, 			// maximum number of repetitions per coin
		int[] F) 		// forbidden coins
	{
		/*------------------- INSERT YOUR CODE HERE -----------------------*/

/*vvvvvvvvvvvvvvvvvvvv DELETE FROM STUDENT CODE vvvvvvvvvvvvvvvvvvvvvv*/
		/*--------------------------------------------------------------
		 * We solve the problem through a recursive search. Let C(n,q,m)
		 * denote the number of ways of making change for the amount q
		 * assuming n coins and the largest coin used has value 1/m. We
		 * assume that n >= 0 and m >= 1. It will be easier to assume
		 * that q is arbitrary, but we will never generate a solution if
		 * q becomes negative. The recursive rule is derived as follows.
		 * Let us assume for now that there are no forbidden values and
		 * any number of repetitions are allowed.
		 *
		 * 1. If n == 0 (no coins), then there are two possibilities. 
		 * 
		 *   a. First, q == 0,in which case the answer is 1, that is,
		 *   we can form the value 0 using 0 coins. The solution is the
		 *   empty set of coins. 
		 *   
		 *   b. If q != 0, then there is no solution, since it is not
		 *   possible to form a nonzero amount from 0 coins.
		 * 
		 * 2. If n >= 1, there are two cases:
		 * 
		 *    a. q <= 0: There is no solution, since we can only form
		 *    positive amounts using 1 or more coins.
		 *    
		 *    b. q > 0: There are two ways that we might form q: 
		 *
		 *      i. We do NOT use the coin 1/m: In this case we must form
		 *      the sum q using n coins of values 1/(m+1) and smaller,
		 *      and so the answer is C(n,q,m+1). 
		 *
		 *      ii. We DO use coin 1/m: In this case, we have n-1 coins
		 *      that remain to form the sum q - (1/m).  Thus, the answer
		 *      is C(n-1, q-(1/m), m).  (We don't explicitly check
		 *      whether the remainder q-(1/m) is negative, since, if it
		 *      is, we will return 0 by rule 2a.)
		 *
		 * Unfortunately, the above recursion could loop indefinitely,
		 * since C(n,q,m) calls C(n,q,m+1) calls C(n,q,m+2), .... To
		 * fix this, we observe that the largest sum that can be formed
		 * by n coins of values at most 1/m is n/m. If n/m < q, we 
		 * return 0. As long as q is positive, eventually, m will be 
		 * small enough that this condition terminates the recursion.
		 *
		 * This leads to the following formula:
		 *
		 *   1.a: C(0,q,m) = 1, if (q = 0)
		 *   1.b: C(0,q,m) = 0, if (q > 0)
		 *
		 * For n >= 1:
		 *   2.a: C(n,q,m) = 0, if (q <= 0)
		 *   2.b.i:  C(n,q,m) = C(n,q,m+1), if (q > 0 and n/m < q)
		 *   2.b.ii: C(n,q,m) = C(n,q,m+1) + C(n-1,q-(1/m),m), otherwise
		 *
		 * We add a few more enhancements:
		 *
		 * - We allow for a set of forbidden coins F. If m is in F, then
		 *   we apply only rule 2.b.i, not 2.b.ii.
		 * 
		 * - We allow each coin to be used at most r times. To do this,
		 *   we add an additional parameter t, so that C(n,q,m,t) is the
		 *   number of coin combinations involving n coins, that sum to
		 *   q assuming that the largest coin is 1/m and it can be used
		 *   at most t times. We add a new rule just before rule 2.b.ii,
		 *   which says that C(n,q,m,0) = C(n,q,m+1,r), which means that
		 *   when we run out of the number of times we can use coin 1/m,
		 *   we immediately jump to coin 1/(m+1), with all r
		 *   repetitions.  If t > 0, then we can use the coin 1/m, but
		 *   we have now have one fewer coin and one fewer allowed
		 *   repetitions of 1/m, and so C(n,q,m,t) = C(n-1, q-(1/m), m,
		 *   t-1).
		 * 
		 * - We can optimize rule 2.b.i by observing that there is no
		 *   point in trying a number m such that 1/m > q. Thus, rather
		 *   than advancing to m+1, we instead define mm to be the
		 *   maximum of m+1 and ceiling(1/q). We use mm as the next
		 *   value. (This is extremely important for efficiency when
		 *   the number of coins is large.)
		 *
		 * To keep track of the solutions, we maintain an ArrayList of
		 * coins, which stores the reciprocals of the coin values used
		 * so far in the partial solution. We push a value onto coins
		 * whenever rule 2.b.ii is applied. When a solution is detected
		 * (in rule 1.a, we save the contents of this list in an
		 * ArrayList of solutions, called sols. When the process is
		 * finished, we convert this to an two dimensional array of
		 * values.
		 *
		 * The call makeChange(n, r, t, F, q, m, coins, result) below
		 * corresponds to C(n, q, m, t) as described above.
		 *--------------------------------------------------------------
		 */
										// partial result list
		ArrayList<Integer> coins = new ArrayList<Integer>();
										// collection of final solutions
		ArrayList<ArrayList<Integer>> sols = new ArrayList<ArrayList<Integer>>();
										// compute the answer recursively
		recurMakeChange(n, r, r, F, 1, 1, coins, sols);
										// convert to array
		int[][] R = new int[sols.size()][];
		for (int i = 0; i < sols.size(); i++) {
			ArrayList<Integer> S = sols.get(i);
			int[] ss = new int[S.size()];
			for (int j = 0; j < S.size(); j++) ss[j] = S.get(j);
			R[i] = ss;
		}
		return R;
	}

	/** recurMakeChange
	 *  
	 * Recursive formulation of makeChange.
	 */
	static void recurMakeChange(		// recursive change counter
		int n,							// number of coins
		int r, 							// max repetitions per coin
		int t,							// number of repetitions remaining
		int[] F, 						// forbidden coins
		double q,						// target sum
		int m,							// 1/m is largest coin to use
		ArrayList<Integer> coins,			// coins of partial solution
		ArrayList<ArrayList<Integer>> sols) // list of solutions
	{
	  									// print debug information
		//System.out.print("[" + n + "," + q + "," + m + "] ");
		//System.out.print("(");
		//for (int i = 0; i < coins.size(); i++) {
		//	if (i != 0) System.out.print(" ");
		//	System.out.print(coins.get(i));
		//}
		//System.out.print(") ");
		if (n == 0) {					// no coins left?
			if (Math.abs(q) < TINY) {	// remainder is 0 -- solution found
				//System.out.println("<------------------Solution found!");
										// save copy of coins
				ArrayList<Integer> cp = new ArrayList<Integer>();
				for (int i = 0; i < coins.size(); i++) {
					cp.add(new Integer(coins.get(i)));
				}
				sols.add(cp);			// add to solution list
			}
			else {
				//System.out.println();
			}
		}
		else {							// at least one coin left?
			//System.out.println();
			if (q > TINY && q*m - n < TINY) { // is solution still possible?
				//System.out.println();
										// find solutions based on using 1/m
				boolean forbid = false;
				for (int i = 0; i < F.length; i++)		// is m forbidden?
					if (m == F[i]) forbid = true;

				if (!forbid && (r == 0 || t > 0)) {		// can we use 1/m?
					coins.add(new Integer(m));			// add m to used-coin list
					recurMakeChange(n-1, r, t-1, F, q - 1/((double) m), m, coins, sols);
					coins.remove(coins.size()-1);		// remove m from list
				}
										// find solutions based on not using 1/m
										// next viable is max(m+1,ceil(1/q))
				int mm = (int) Math.max(m+1, Math.ceil(1/(q + TINY)));
				//int mm = m+1;
				recurMakeChange(n, r, r, F, q, mm, coins, sols);
			}
			//else System.out.println("Infeasible");
		}
		//System.out.println();

/*^^^^^^^^^^^^^^^^^^^^^^^^^^^ END OF DELETION ^^^^^^^^^^^^^^^^^^^^^^^^*/
	}
}
